@@ -13,6 +13,7 @@ from traceback import TracebackException
 from typing import Callable
 import sys
 import re
+import time
 
 
 PREFIX = "SystemLog:"
@@ -66,6 +67,7 @@ def print_prefixed_stack_trace_and_raise(
     scrub_message: str = SCRUB_MESSAGE,
     keep_message: bool = False,
     allow_list: list = [],
+    add_timestamp: bool = False,
     err: BaseException = None,
 ) -> None:
     """
@@ -93,7 +95,11 @@ def print_prefixed_stack_trace_and_raise(
             continue
         lines = execution.splitlines()
         for line in lines:
-            print(f"{prefix} {line}", file=file)
+            if add_timestamp:
+                current_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
+                print(f"{prefix} {current_time} {line}", file=file)
+            else:
+                print(f"{prefix} {line}", file=file)
 
     # raise compliant error
     if not err:
@@ -109,6 +115,57 @@ def print_prefixed_stack_trace_and_raise(
         raise type(err)(message) from err
 
 
+class _PrefixStackTraceWrapper:
+    """
+    Callable object for catching exceptions and printing their stack traces,
+    appropriately prefixed.
+
+    This is an object instead of a nested function to support working in Spark.
+    Python anonymous functions are not "pickleable", so we need to define a
+    class which handles this logic, so the class is picklable.
+    """
+
+    def __init__(
+        self,
+        file: io.TextIOBase,
+        disable: bool,
+        prefix: str,
+        scrub_message: str,
+        keep_message: bool,
+        allow_list: list,
+        add_timestamp: bool,
+    ) -> None:
+        self.allow_list = allow_list
+        self.disable = disable
+        self.file = file
+        self.keep_message = keep_message
+        self.prefix = prefix
+        self.scrub_message = scrub_message
+        self.add_timestamp = add_timestamp
+
+    def __call__(self, function) -> Callable:
+        @functools.wraps(function)
+        def wrapper(*func_args, **func_kwargs):
+            """
+            Create a wrapper which catches exceptions thrown by `function`,
+            scrub exception messages, and logs the prefixed stack trace.
+            """
+            try:
+                return function(*func_args, **func_kwargs)
+            except BaseException as err:
+                print_prefixed_stack_trace_and_raise(
+                    self.file,
+                    self.prefix,
+                    self.scrub_message,
+                    self.keep_message,
+                    self.allow_list,
+                    self.add_timestamp,
+                    err,
+                )
+
+        return function if self.disable else wrapper
+
+
 def prefix_stack_trace(
     file: io.TextIOBase = sys.stderr,
     disable: bool = sys.flags.debug,
@@ -116,6 +173,7 @@ def prefix_stack_trace(
     scrub_message: str = SCRUB_MESSAGE,
     keep_message: bool = False,
     allow_list: list = [],
+    add_timestamp: bool = False,
 ) -> Callable:
     """
     Decorator which wraps the decorated function and prints the stack trace of
@@ -128,29 +186,9 @@ def prefix_stack_trace(
             pass
     """
 
-    def decorator(function: Callable) -> Callable:
-        """
-        Create a decorator to catch, modify, and log an exception with its
-        stack trace. Follows:
-        https://www.blog.pythonlibrary.org/2016/06/09/python-how-to-create-an-exception-logging-decorator/
-        """
-
-        @functools.wraps(function)
-        def wrapper(*func_args, **func_kwargs):
-            """
-            Create a wrapper which catches exceptions thrown by `function`,
-            scrub exception messages, and logs the prefixed stack trace.
-            """
-            try:
-                return function(*func_args, **func_kwargs)
-            except BaseException as err:
-                print_prefixed_stack_trace_and_raise(
-                    file, prefix, scrub_message, keep_message, allow_list, err
-                )
-
-        return function if disable else wrapper
-
-    return decorator
+    return _PrefixStackTraceWrapper(
+        file, disable, prefix, scrub_message, keep_message, allow_list, add_timestamp
+    )
 
 
 class PrefixStackTrace:
@@ -161,12 +199,16 @@ class PrefixStackTrace:
         prefix: str = PREFIX,
         scrub_message: str = SCRUB_MESSAGE,
         keep_message: bool = False,
+        add_timestamp: bool = False,
+        allow_list: list = [],
     ):
         self.file = file
         self.disable = disable
         self.prefix = prefix
         self.scrub_message = scrub_message
         self.keep_message = keep_message
+        self.add_timestamp = add_timestamp
+        self.allow_list = allow_list
 
     def __enter__(self):
         pass
@@ -174,9 +216,11 @@ class PrefixStackTrace:
     def __exit__(self, exc_type, exc_value, traceback):
         if exc_type and not self.disable:
             print_prefixed_stack_trace_and_raise(
-                self.file,
-                self.prefix,
-                self.scrub_message,
-                self.keep_message,
+                file=self.file,
+                prefix=self.prefix,
+                scrub_message=self.scrub_message,
+                keep_message=self.keep_message,
+                allow_list=self.allow_list,
+                add_timestamp=self.add_timestamp,
                 err=exc_value,
             )
